import { computed } from 'vue';

export interface EnumProps {
    enum?: string[];
    enumMapperKey?: string;
    disableEnumMapping?: boolean;
    enumMapper?: Record<string, string>;
}

const enumMap: Record<string, Record<string, string>> = {};

export function useEnum({ enum: enumList = [], enumMapperKey = 'default', disableEnumMapping, enumMapper }: EnumProps) {
    const mappedEnum = computed<Record<string, string>>(() => {
        if (enumMapper) return enumMapper;
        let result = enumList.reduce((acc, item) => ({ ...acc, [item]: item }), {});
        if (!disableEnumMapping && enumMap?.[enumMapperKey]) result = enumMap[enumMapperKey];
        return result;
    });
    return { mappedEnum };
}

export function setEnumMap(key: string = 'default', value: Record<string, string>) {
    enumMap[key] = value;
}
