import { deepAssign } from '@codeperate/utils';
import { defineAsyncComponent, markRaw } from 'vue';
import { ArrayWidgetProps } from '../ArrayWidget';
import { BooleanWidgetProps } from '../BooleanWidget';
import { CheckBoxWidgetProps } from '../CheckBoxWidget';
import { DateTimeWidgetProps } from '../DateTimeWidget';
import { DateWidgetProps } from '../DateWidget';
import { EnumProps } from '../enum';
import { FileWidgetProp } from '../FileWidget';
import { FormSchema, IWidget } from '../FormBuilder';
import { NumberWidgetProps } from '../NumberWidget';
import { ObjectWidgetProps } from '../ObjectWidget';
import { PasswordWidgetProps } from '../PasswordWidget';
import { SectionWidgetProps } from '../SectionWidget';
import { StringWidgetProps } from '../StringWidget';
import { TextAreaWidgetProps } from '../TextAreaWidget';
import { RangeWidgetProps } from '../RangeWidget';

export function defineWidget<T, V>(widget: IWidget<T, V>) {
    return widget;
}

export const StringWidget = defineWidget<string, StringWidgetProps & EnumProps>({
    component: markRaw(defineAsyncComponent(() => import('../StringWidget/StringWidget.vue'))),
    columns: 6,
});

export const ObjectWidget = defineWidget<object, ObjectWidgetProps>({
    component: markRaw(defineAsyncComponent(() => import('../ObjectWidget/ObjectWidget.vue'))),
    columns: 12,
});

export const TextAreaWidget = defineWidget<string, TextAreaWidgetProps>({
    component: markRaw(defineAsyncComponent(() => import('../TextAreaWidget/TextAreaWidget.vue'))),
    columns: 12,
});

export const PasswordWidget = defineWidget<string, PasswordWidgetProps>({
    component: markRaw(defineAsyncComponent(() => import('../PasswordWidget/PasswordWidget.vue'))),
    columns: 12,
});

export const ArrayWidget = defineWidget<any[], ArrayWidgetProps>({
    component: markRaw(defineAsyncComponent(() => import('../ArrayWidget/ArrayWidget.vue'))),
    columns: 12,
});

export const NumberWidget = defineWidget<number, NumberWidgetProps>({
    component: markRaw(defineAsyncComponent(() => import('../NumberWidget/NumberWidget.vue'))),
    columns: 6,
});

export const BooleanWidget = defineWidget<boolean, BooleanWidgetProps>({
    component: markRaw(defineAsyncComponent(() => import('../BooleanWidget/BooleanWidget.vue'))),
    columns: 6,
});

export const DateWidget = defineWidget<string, DateWidgetProps>({
    component: markRaw(defineAsyncComponent(() => import('../DateWidget/DateWidget.vue'))),
    columns: 6,
});

export const DateTimeWidget = defineWidget<string, DateTimeWidgetProps>({
    component: markRaw(defineAsyncComponent(() => import('../DateTimeWidget/DateTimeWidget.vue'))),
    columns: 6,
});

export const SectionWidget = defineWidget<any, SectionWidgetProps>({
    component: markRaw(defineAsyncComponent(() => import('../SectionWidget/SectionWidget.vue'))),
    columns: 12,
});
export const Section = (title: string, formSchema: Partial<FormSchema> = {}, key?: number | string | symbol) => {
    const obj = deepAssign({ widget: SectionWidget, props: { title }, label: false }, formSchema);
    return { [key ?? title]: obj };
};

export const FileWidget = defineWidget<File, FileWidgetProp>({
    component: markRaw(defineAsyncComponent(() => import('../FileWidget/FileWidget.vue'))),
    columns: 12,
});

export const CheckBoxWidget = defineWidget<string[], CheckBoxWidgetProps & EnumProps>({
    component: markRaw(defineAsyncComponent(() => import('../CheckBoxWidget/CheckBoxWidget.vue'))),
    columns: 12,
});

export const RangeWidget = defineWidget<number, RangeWidgetProps>({
    component: markRaw(defineAsyncComponent(() => import('../RangeWidget/RangeWidget.vue'))),
    columns: 6,
});
